// Copyright (c) 2022, Pritam Hazra.

import 'dart:convert';
import 'dart:typed_data';
import 'package:pointycastle/export.dart';
import '../crypto_types.dart';

Converter<List<int>, List<int>>? hashConverter(GRCryptoType type) {
  Digest? digest = _getDigestFor(type);
  if (digest == null) {
    return null;
  }
  return _CustomConverter(digest);
}

Uint8List convertFrom(
    List<String> input, Converter<List<int>, List<int>> converter) {
  final conv = converter as _CustomConverter;
  for (String element in input) {
    final chunk = utf8.encode(element);
    conv.digest.update(chunk as Uint8List, 0, chunk.length);
  }
  final hash = Uint8List(converter.digest.digestSize);
  conv.digest.doFinal(hash, 0);
  return hash;
}

class _CustomConverter extends Converter<Uint8List, List<int>> {
  Digest digest;
  _CustomConverter(this.digest);
  @override
  Uint8List convert(Uint8List input) {
    return this.digest.process(input);
  }
}

Digest? _getDigestFor(GRCryptoType type) {
  switch (type) {
    case GRCryptoType.md2:
      return MD2Digest();
    case GRCryptoType.md4:
      return MD4Digest();
    // case GRCryptoType.blake2b:
    //   return Blake2bDigest();
    case GRCryptoType.ripemd128:
      return RIPEMD128Digest();
    case GRCryptoType.ripemd160:
      return RIPEMD160Digest();
    case GRCryptoType.ripemd256:
      return RIPEMD256Digest();
    case GRCryptoType.ripemd320:
      return RIPEMD320Digest();
    case GRCryptoType.sha3_224:
      return SHA3Digest(224);
    case GRCryptoType.sha3_256:
      return SHA3Digest(256);
    case GRCryptoType.sha3_384:
      return SHA3Digest(384);
    case GRCryptoType.sha3_512:
      return SHA3Digest(512);
    case GRCryptoType.keccak224:
      return KeccakDigest(224);
    case GRCryptoType.keccak256:
      return KeccakDigest(256);
    case GRCryptoType.keccak384:
      return KeccakDigest(384);
    case GRCryptoType.keccak512:
      return KeccakDigest(512);
    case GRCryptoType.tiger:
      return TigerDigest();
    case GRCryptoType.whirlpool:
      return WhirlpoolDigest();
    case GRCryptoType.sm3:
      return SM3Digest();
    default:
      return null;
  }
}
